#include "bezier.h"

#include <GL/glew.h>
#include <iostream>
#include <cstdlib>

Bezier::Bezier(void){
    max_points_=4;
    num_points_=0;
}

Bezier::~Bezier(){
    glDisableVertexAttribArray(0);
    glBindBuffer(GL_ARRAY_BUFFER, 0);
    glDeleteBuffers(1, &vertex_buffer_);

    glBindVertexArray(0);
    glDeleteVertexArrays(1,&vao_);

}

void Bezier::Initialize(){
    glGenVertexArrays(1, &vao_);
    glBindVertexArray(vao_);

    glGenBuffers(1, &vertex_buffer_);
    glBindBuffer(GL_ARRAY_BUFFER, vertex_buffer_);
    glBufferData(GL_ARRAY_BUFFER, max_points_*2*sizeof(GLfloat), NULL, GL_DYNAMIC_DRAW);
    glVertexAttribPointer(0, 2, GL_FLOAT, GL_FALSE, 0, (GLvoid*)0);
    glEnableVertexAttribArray(0);

    glBindVertexArray(0);
}



void Bezier::Draw(const BezierProgram &program){

    glBindVertexArray(vao_);
    glUseProgram( program.GetLineProgram() );

    glPointSize(8);
    glDrawArrays(GL_POINTS, 0, num_points_);
    glPointSize(1);
    
    glDrawArrays(GL_LINE_STRIP, 0, num_points_);
    if(num_points_ == max_points_){
        glUseProgram( program.GetBezierProgram() );
        glDrawArrays(GL_LINES_ADJACENCY, 0, num_points_);
    }

    glBindVertexArray(0);
    glUseProgram(0);
}

void Bezier::AddPoint( GLfloat*v ){
    if(num_points_ < max_points_){
      glBindVertexArray(vao_);
      glBindBuffer(GL_ARRAY_BUFFER, vertex_buffer_);
      glBufferSubData(GL_ARRAY_BUFFER, (num_points_++)*2*sizeof(GLfloat), 2*sizeof(GLfloat), v);
      glBindVertexArray(0);
    }
}
